function [eventVariables, crashDetection]= EventGenerator(Vego, Vneighbour)
% INPUT: Initial velocity of ego and neighbour 
 
% EVENT :
% * Ego and Neighbour moves at a constant longitudinal velocity 
% * leader brakes at 6 sec
% * Lane change duration is 3.5 sec
% * Lateral velocity during lane change is 1 m/s;
% * Lane width =3.5 m.

% OUTPUT: 
% * This function returns the trajectories of the ego when a
%   neighbour cuts in, with a resolution of 0.1 s for a duration of 10 s

% eventVariable 
% This matrix contain [100 x 8] cells and each column depicts the follows:
% 1) X: Longitudinal position of subject in m
% 2) Y: Lateral position of subject in m
% 3) VX: Longitudinal velocity of subject in m/s
% 4) VY: Lateral velocity of subject in m/s
% 5) Longitudinal position of neighbour in m
% 6) Lateral position of neighbour in m
% 7) Longitudinal velocity of neighbour in m/s
% 8) Lateral velocity of neighbour in m/s

% EGO(0,3.5)------> NEIGHBOUR(80,3.5)
%



%Initialize state of ego vehicle
eventVariables(1,1)=0;eventVariables(1,2)=3.5;eventVariables(1,3)=Vego;eventVariables(1,4)=0;
%Initialize state of neighbour vehicle
eventVariables(1,5)=60;eventVariables(1,6)=3.5;eventVariables(1,7)=Vneighbour;eventVariables(1,8)=0;
% set timestep
dt = 0.1;

% set total time 
T = 15;
H = T/dt;

% set lane change start and end time 
bk_start =6/dt;
bk_end = bk_start+ (Vneighbour/5/dt);
% initialize crash detection
crashDetection(1,1)=0;crashDetection(2,1)=0;

% Propogate the ego vehicle 
for i = 2:H
   % set velocity as constant
   eventVariables(i,3)= eventVariables(i-1,3); 
   eventVariables(i,4)= 0;%eventVariables(i-1,4);
   % propogate positions
   eventVariables(i,1) = eventVariables(i-1,1)+eventVariables(i-1,3)*dt;
   eventVariables(i,2) = 3.5;%eventVariables(i-1,2)+eventVariables(i-1,4)*dt;
end 

% Propogate the ego vehicle 
% for i = 2:H
%    % set velocity as constant
%    eventVariables(i,3)= eventVariables(i-1,3); 
%    eventVariables(i,4)= eventVariables(i-1,4);
%    
%    % propogate positions
%    eventVariables(i,1) = eventVariables(i-1,1)+eventVariables(i-1,3)*dt;
%    eventVariables(i,2) = eventVariables(i-1,2)+eventVariables(i-1,4)*dt;
% end 

% Propogate the neighbour vehicle 
for i = 2:H
   % set longitudinal velocity as constant
%    eventVariables(i,3+4)= eventVariables(i-1,3+4);
   % set longitudianl velocity 
   if i <= bk_start
      eventVariables(i,7)=eventVariables(i-1,7);
   elseif (i>bk_start)&&(i<bk_end)
      eventVariables(i,7)=eventVariables(i-1,7)-0.5;
   else
      eventVariables(i,7)=0;
   end 

   % set lateral velocity as 0
    eventVariables(i,4+4)=0;
   % propogate positions
   eventVariables(i,1+4) = eventVariables(i-1,5)+eventVariables(i-1,7)*dt;
   eventVariables(i,2+4) = 3.5;%eventVariables(i-1,2+4)+eventVariables(i-1,4+4)*dt;
   
   % crashDetection 
   if (crashDetection(1,1)==0)&&(abs(eventVariables(i,1)- eventVariables(i,1+4))< 5 ) && (abs(eventVariables(i,2)- eventVariables(i,2+4))<2)
        crashDetection(1,1)=1;
        crashDetection(2,1)=i*0.1;
   end
end 

%{
timestamp = linspace(0.1,10,100);
figure 
subplot(2,1,1)
plot(timestamp,eventVariables(:,1),timestamp,eventVariables(:,1+4))
subplot(2,1,2)
plot(eventVariables(:,1),eventVariables(:,2),eventVariables(:,1+4),eventVariables(:,2+4))
%}
end 
