# Usage 
1. Place the four scripts: `MAIN.m`, `C_PROB.m`, `KINETIC_PDRF.m`, `reachable.m` and `EventGenerator.m` , in the same subfolder 
2. Run `MAIN.m` from MATLAB.

# Description of the scripts 
These scripts were used to create `Figure 8` and `Figure 9` in the following article:

`Mullakkal-Babu, Freddy A., et al. "Probabilistic field approach for motorway driving risk assessment." Transportation research part C: emerging technologies 118 (2020): 102716.`


## MAIN.m
This is the upper level funtion that defines the experiment settings (parameter range and sampling interval) and the output plots. This function calls `KINETIC_PDRF.m` with input parameter values as the argument. In this case the two input parameters are initial velocity of the ego vehicle and the cut-in/leading vehicle. 

## EventGenerator.m
This function generates the trajectories of two vehicles involved the in a cut-in encounter: ego vehicle and cut-in/leading vehicle.

### Input variables
+ Initial velocity of ego vehicle
+ Initial velocity of cut-in/leading vehicle 

### Output variables :
+ crashDetection: This is a binary variable, 0 implies no -crash and 1 implies crash.
+ eventVariable
    + This is a [100 x 8] matrix. Each column is a time-series of resolution of 0.1 s. The variable listed in the columns are 
1. X: Longitudinal position of subject in m
2. Y: Lateral position of subject in m
3. VX: Longitudinal velocity of subject in m/s
4. VY: Lateral velocity of subject in m/s
5. Longitudinal position of neighbour in m
6. Lateral position of neighbour in m
7. Longitudinal velocity of neighbour in m/s
8. Lateral velocity of neighbour in m/s

### Event description 
+ Ego and Neighbour moves at a constant longitudinal velocity
+ Neighbour cuts-in or hard-brakes at 6 sec
+ Duration of the cut-in maneouvre is 3.5 sec 
+ Lateral velocity during lane change is 1 m/s;
+ Lane width =3.5 m.

## KINETIC_PDRF.m
+ This calculates the single-step PDRF risk of the ego with neighbouring vehicle. 
+ This function call `EventGenerator.m` to obtain trajectories of the two vehicles. 
+ This function calls two helper functions: `C_PROB.m` to estimate the single-step PDRF

## C_PROB.m
This function calculates the crash probability between two vehicles at a given future time step. This function calls the helper `reachable.m` to estimate the reachable region of a vehicle. 

## reachable.m
This function calculates the reachable region of a vehicle at a future time step, based on its initial  kinematic state and vehicle dynamics contraints. 
