% This script calculates the collision probability of the subject vehicle.
% The trajectory of the subject vehicle is defined here
% The set of reachable positions of the NEIGHBOURING VEHICLE
% and probability associated with position are fetched from the POSITION.m
% function
T = 5; % planning horizon in seconds
tk = 1; % time step in seconds
ac = 5; % number of discrete time steps
ac2 = ac * ac;

%initialisation of the subject vehicle state
X = zeros(1, T/tk+1);
X(1, 1) = 15;
Y = zeros(1, T/tk+1);
Y(1, 1) = 0;
VX = zeros(1, T/tk+1);
VX(1, 1) = 20;
VY = zeros(1, T/tk+1);
VY(1, 1) = 1;
AX = zeros(1, T/tk+1);
AY = zeros(1, T/tk+1);
X1 = zeros(1, ac);
X2 = zeros(ac^(2 - 1), ac);
X3 = zeros(ac^(3 - 1), ac);
X4 = zeros(ac^(4 - 1), ac);
X5 = zeros(ac^(5 - 1), ac);
PX1 = zeros(1, ac);
PX2 = zeros(ac^(2 - 1), ac);
PX3 = zeros(ac^(3 - 1), ac);
PX4 = zeros(ac^(4 - 1), ac);
PX5 = zeros(ac^(5 - 1), ac);
V1 = zeros(1, ac);
V2 = zeros(ac^(2 - 1), ac);
V3 = zeros(ac^(3 - 1), ac);
V4 = zeros(ac^(4 - 1), ac);
V5 = zeros(ac^(5 - 1), ac);

%obtaining the surrounding vehicle reachability information
[~,YC,POX1, POX2, POX3, POX4, POX5, POY1, POY2, POY3, POY4, POY5, PVX1, PVX2, PVX3, PVX4, PVX5, PVY1, PVY2, PVY3, PVY4, PVY5, PP1, PP2, PP3, PP4, PP5] = POSITION;
[XC,X1, X2, X3, X4, X5, PX1, PX2, PX3, PX4, PX5, V1, V2, V3, V4, V5] = TR_X5;

%Initialsing few local variables
CPP1 = zeros(1, ac);
CPP2 = zeros(ac^2, ac^2);
CPP3 = zeros(ac^4, ac^2);
CPP4 = zeros(ac^6, ac^2);
NCPP1 = zeros(1, ac);
NCPP2 = zeros(ac^2, ac^2);
NCPP3 = zeros(ac^4, ac^2);
NCPP4 = zeros(ac^6, ac^2);


%definition of subject vehicle acceleration control
for q = 1:T / tk + 1
    AX(1, q) = -2;
    AY(1, q) = 0;
end

%definition of subject vehicle trajectory
for s = 2:(T / tk + 1)
    X(1, s) = X(1, s-1) + (VX(1, s - 1) * tk) + (0.5 * AX(1, s - 1) * tk * tk);
    VX(1, s) = VX(1, s-1) + (AX(1, s - 1) * tk);
    Y(1, s) = Y(1, s-1) + (VY(1, s - 1) * tk) + (0.5 * AY(1, s - 1) * tk * tk);
    VY(1, s) = VY(1, s-1) + (AY(1, s - 1) * tk);
end

%check for collision in tk =1 and calculate the collision probability
for c = 1:ac * ac
    if ((X(1, 2)-POX1(1, c)) < 4) && (POY1(1, c) - (Y(1, 2)) < 2.4)
        CPP1(1, c) = PP1(1, c);
        NCPP1(1, c) = PP1(1, c);
    else
        CPP1(1, c) = 0;
        NCPP1(1, c) = 0;
    end
end

%check for collision in tk =2 and calculate the collision probability

for r = 1:ac * ac
    for c = 1:ac2
        if rem(r, ac2) > 0
            rm(r, c) = (ac2) - rem(r, ac2);
        else
            rm(r, c) = 0;
        end

        if ((X(1, 3) - POX2(r, c)) < 4) && (CPP1(1, ((ac2) - rm(r, c))) == 0) && ((POY2(r, c) - Y(1, 3)) < 2.4)
            CPP2(r, c) = PP2(r, c);
            NCPP2(r, c) = PP2(r, c);
        elseif (CPP1(1, ac2 - rm(r, c)) > 0)
            CPP2(r, c) = 1;
            NCPP2(r, c) = 0;
        else
            CPP2(r, c) = 0;
            NCPP2(r, c) = 0;
        end

    end
end

%check for collision in tk =3 and calculate the collision probability

for r = 1:ac * ac * ac * ac
    for c = 1:ac2
        if rem(r, ac2) > 0
            rm3(r, c) = ac2 - rem(r, ac2);
        else
            rm3(r, c) = 0;
        end

        if ((X(1, 4) - POX3(r, c)) < 4) && ((POY3(r, c)) - Y(1, 4) < 2.4) && (CPP2(((r + rm3(r, c))/ac2), (ac2 - rm3(r, c))) == 0)
            %test(r,c)=1;
            CPP3(r, c) = PP3(r, c);
            NCPP3(r, c) = PP3(r, c);
        elseif (CPP2(((r + rm3(r, c))/ac2), (ac2 - rm3(r, c))) > 0)
            CPP3(r, c) = 1;
            NCPP3(r, c) = 0;

        else
            CPP3(r, c) = 0;
            NCPP3(r, c) = 0;

        end

    end
end
%check for collision in tk =4 and calculate the overlap probability
%associated with the time step

for r = 1:ac * ac * ac * ac * ac * ac
    for c = 1:ac2
        if rem(r, ac2) > 0
            rm4(r, c) = ac2 - rem(r, ac2);
        else
            rm4(r, c) = 0;
        end
        if ((X(1, 5) - POX4(r, c)) < 4) && ((POY4(r, c)) - Y(1, 5) < 2.4) && (CPP3(((r + rm4(r, c))/ac2), (ac2 - rm4(r, c))) == 0)
            CPP4(r, c) = PP4(r, c);
            NCPP4(r, c) = PP4(r, c);
        elseif (CPP3(((r + rm4(r, c))/ac2), ac2 - rm4(r, c)) > 0)
            CPP4(r, c) = 1;
            NCPP4(r, c) = 0;
        else
            CPP4(r, c) = 0;
            NCPP4(r, c) = 0;
        end
    end
end

% estimate the collision probability at each time step
CP1 = sum(sum(NCPP1));
CP2 = sum(sum(NCPP2)) * (1 - sum(sum(NCPP1)));
CP3 = sum(sum(NCPP3)) * (1 - sum(sum(NCPP2))) * (1 - sum(sum(NCPP1)));
CP4 = sum(sum(NCPP4)) * (1 - sum(sum(NCPP3))) * (1 - sum(sum(NCPP2))) * (1 - sum(sum(NCPP1)));
CP5 = 0;

SCP1 = sum(sum(NCPP1));
SCP2 = sum(sum(NCPP2));
SCP3 = sum(sum(NCPP3));
SCP4 = sum(sum(NCPP4));
SCP5 = 0;
TCP = CP1 + CP2 + CP3 + CP4 + CP5;
SCP = SCP1 + SCP2 + SCP3 + SCP4 + SCP5;

figure;%figure 3
hold on;
plot(POX1, POY1,'k.')
plot(POX2, POY2,'k.')
plot(POX3, POY3,'k.')
plot(POX4, POY4,'k.')
plot(XC(1:5),YC(1:5),'ko-',...
                    'MarkerFaceColor','g',...
                    'MarkerEdgeColor','k',...
                    'MarkerSize',8);
set(gca,'XLim',[0 100],'XTick',20*(0:5), ...
        'YLim',[-20 20],'YTick',10*(-2:2))
xlim([0 100]);
ylim([-20, 20]);
xlabel('Longitudinal position');
ylabel({'Lateral position'});
