function [XC,YC,POX1, POX2, POX3, POX4, POX5, POY1, POY2, POY3, POY4, POY5, PVX1, PVX2, PVX3, PVX4, PVX5, PVY1, PVY2, PVY3, PVY4, PVY5, PP1, PP2, PP3, PP4, PP5] = POSITION
% This script combines and restructures the predction matrices from TR_X5 and TR_Y5, so
% that they are easier to be manipulated during collision probability
% estimation.
% During combination, some predicted states are re-assigned zero probability
% the filtered states are the ones infeasible based on non-holonomic constraints
% fetch predicted vehicle states and probabilities along  X and Y axis
[XC,X1, X2, X3, X4, X5, PX1, PX2, PX3, PX4, PX5, V1, V2, V3, V4, V5] = TR_X5;
[YC,Y1, Y2, Y3, Y4, Y5, PY1, PY2, PY3, PY4, PY5, VY1, VY2, VY3, VY4, VY5] = TR_Y5;
ac = 5; % number of discrete time steps

POX1 = zeros(1, ac*ac);
POY1 = zeros(1, ac*ac);
PP1 = zeros(1, ac*ac);
POX2 = zeros(ac^(2), ac*ac);
POY2 = zeros(ac^(2), ac*ac);
PP2 = zeros(ac^(2), ac*ac); 
POX3 = zeros(ac^(4), ac*ac);
POY3 = zeros(ac^(4), ac*ac);
PP3 = zeros(ac^(4), ac*ac);
POX4 = zeros(ac^(6), ac*ac);
POY4 = zeros(ac^(6), ac*ac);
PP4 = zeros(ac^(6), ac*ac);
POX5 = zeros(ac^(8), ac*ac);
POY5 = zeros(ac^(8), ac*ac);
PP5 = zeros(ac^(8), ac*ac);
PVX1 = zeros(1, ac*ac);
PVY1 = zeros(1, ac*ac);
PVX2 = zeros(ac^(2), ac*ac);
PVY2 = zeros(ac^(2), ac*ac);
PVX3 = zeros(ac^(4), ac*ac);
PVY3 = zeros(ac^(4), ac*ac);
PVX4 = zeros(ac^(6), ac*ac);
PVY4 = zeros(ac^(6), ac*ac);
PVX5 = zeros(ac^(8), ac*ac);
PVY5 = zeros(ac^(8), ac*ac);

%position set for tk=1
for r = 1:ac
    for c = 1:ac
        POX1(1, (r - 1)*ac+c) = X1(1, r); % restructured position matrix
        POY1(1, (r - 1)*ac+c) = Y1(1, c); % restructured position matrix
        if (abs(VY1(1, c)) > abs(0.17 .* V1(1, r))) %non-holonomic constraints
            PY1(1, c) = 0;
        end
        PP1(1, (r - 1)*ac+c) = PX1(1, r) * PY1(1, c); % probability associated with a predicted position
        PVX1(1, (r - 1)*ac+c) = V1(1, r);
        PVY1(1, (r - 1)*ac+c) = VY1(1, c);
    end
end
%position set for tk=2
for r = 1:ac^2
    for k = 1:ac
        for c = 1:ac
            if rem(r, ac) > 0
                rm = (ac) - rem(r, ac);
                rc = rem(r, ac);
            else
                rm = 0;
                rc = ac;
            end
            if (abs(VY2(rc, c)) > abs(0.17 .* V2(((r + rm)/ac), k))) %non-holonomic constraints
                PY2(rc, c) = 0;
            end
            POX2(r, (k - 1)*ac+c) = X2(((r + rm)/ac), k);
            POY2(r, (k - 1)*ac+c) = Y2(rc, c);
            PP2(r, (k - 1)*ac+c) = PX2(((r + rm)/ac), k) * PY2(rc, c);
            PVX2(r, (k - 1)*ac+c) = V2(((r + rm)/ac), k);
            PVY2(r, (k - 1)*ac+c) = VY2(rc, c);
        end
    end
end
%position set for tk=3
for r = 1:ac^4
    for k = 1:ac
        for c = 1:ac
            if rem(r, ac*ac) > 0
                rm = (ac * ac) - rem(r, ac*ac);
                rc = rem(r, ac*ac);
            else
                rm = 0;
                rc = ac * ac;
            end
            if (abs(VY3(rc, c)) > abs(0.17 .* V3(((r + rm)/(ac * ac)), k))) %non-holonomic constraints
                PY3(rc, c) = 0;
            end
            POX3(r, (k - 1)*ac+c) = X3(((r + rm)/(ac * ac)), k);
            POY3(r, (k - 1)*ac+c) = Y3(rc, c);
            PP3(r, (k - 1)*ac+c) = PX3(((r + rm)/(ac * ac)), k) * PY3(rc, c);
            PVX3(r, (k - 1)*ac+c) = V3(((r + rm)/(ac * ac)), k);
            PVY3(r, (k - 1)*ac+c) = VY3(rc, c);
        end
    end
end
%position set for tk=4
for r = 1:ac^6
    for k = 1:ac
        for c = 1:ac
            if rem(r, ac*ac*ac) > 0
                rm = (ac * ac * ac) - rem(r, ac*ac*ac);
                rc = rem(r, ac*ac*ac);
            else
                rm = 0;
                rc = ac * ac * ac;
            end
            if (abs(VY4(rc, c)) > abs(0.17 .* V4(((r + rm)/(ac * ac * ac)), k))) %non-holonomic constraints
                PY4(rc, c) = 0;
            end
            POX4(r, (k - 1)*ac+c) = X4(((r + rm)/(ac * ac * ac)), k);
            POY4(r, (k - 1)*ac+c) = Y4(rc, c);
            PP4(r, (k - 1)*ac+c) = PX4(((r + rm)/(ac * ac * ac)), k) * PY4(rc, c);
            PVX4(r, (k - 1)*ac+c) = V4(((r + rm)/(ac * ac * ac)), k);
            PVY4(r, (k - 1)*ac+c) = VY4(rc, c);
        end
    end
end
%position set for tk=5
for r = 1:ac^8
    for k = 1:ac
        for c = 1:ac
            if rem(r, ac*ac*ac*ac) > 0
                rm = (ac * ac * ac * ac) - rem(r, ac*ac*ac*ac);
                rc = rem(r, ac*ac*ac*ac);
            else
                rm = 0;
                rc = ac * ac * ac * ac;
            end
            if (abs(VY5(rc, c)) > abs(0.17 .* V5(((r + rm)/(ac * ac * ac * ac)), k))) %non-holonomic constraints
                PY5(rc, c) = 0;
            end

            POX5(r, (k - 1)*ac+c) = X5(((r + rm)/(ac * ac * ac * ac)), k);
            POY5(r, (k - 1)*ac+c) = Y5(rc, c);
            PP5(r, (k - 1)*ac+c) = PX5(((r + rm)/(ac * ac * ac * ac)), k) * PY5(rc, c);
            PVX5(r, (k - 1)*ac+c) = V5(((r + rm)/(ac * ac * ac * ac)), k);
            PVY5(r, (k - 1)*ac+c) = VY5(rc, c);
        end
    end
end
end