function [XC,X1, X2, X3, X4, X5, PX1, PX2, PX3, PX4, PX5, V1, V2, V3, V4, V5] = TR_X5
% This function throws back the a matrix of X coordinate of
% future positions of the NEIGHBOURING VEHICLE
%within the time horizon f 5 seconds, and a matrix of associated
% discrete probability

ac = 5; % number of discrete time steps
tk = 1; % time step in seconds

% Initialising the matrices containing the X coordinates of
% future vehicle positions at discrete time steps (1,2,3,..5)
X0 = zeros(1, 1);
X1 = zeros(1, ac);
X2 = zeros(ac^(2 - 1), ac);
X3 = zeros(ac^(3 - 1), ac);
X4 = zeros(ac^(4 - 1), ac);
X5 = zeros(ac^(5 - 1), ac);
j = zeros(ac^(3 - 1), ac);
l = zeros(ac^(3 - 1), ac);

% Initialising the matrices containing the velocity along X axis at
% future vehicle positions at discrete time steps (1,2,3,..5)
V0 = zeros(1, 1);
V1 = zeros(1, ac);
V2 = zeros(ac^(2 - 1), ac);
V3 = zeros(ac^(3 - 1), ac);
V4 = zeros(ac^(4 - 1), ac);
V5 = zeros(ac^(5 - 1), ac);

% Initialising the matrices containing the probabilities of
% future vehicle states at discrete time steps (1,2,3,..5)
PX0 = zeros(1, 1);
PX1 = zeros(1, ac);
PX2 = zeros(ac^(2 - 1), ac);
PX3 = zeros(ac^(3 - 1), ac);
PX4 = zeros(ac^(4 - 1), ac);
PX5 = zeros(ac^(5 - 1), ac);

A = zeros(5, ac);
sig = 0.7; % standard deviation of the discretised normal distribution
sig3 = 3 * sig;

% discrete acceleration input AX
A(:, 3) = 0;
A(:, 1) = A(:, 3) - sig3;
A(:, 2) = A(:, 3) - sig;
A(:, 4) = A(:, 3) + sig;
A(:, 5) = A(:, 3) + sig3;


% Probability associated with each discrete acceleration input,
% derived from discretized normal distribution
PA = zeros(5, ac);
PA(:, 1) = 0.004;
PA(:, 2) = 0.24;
PA(:, 3) = 0.39;
PA(:, 4) = 0.24;
PA(:, 5) = 0.004;

% Intial vehicle state
X0(1, 1) = 0;
V0(1, 1) = 20;
%state prediction for tk=1
for c = 1:ac
    X1(1, c) = X0(1, 1) + V0(1, 1) * tk + 0.5 * A(1, c) * tk * tk;
    PX1(1, c) = PA(1, c);
end
% V prediction for tk=1
for c = 1:ac
    V1(1, c) = V0(1, 1) + A(1, c) * tk;
end

%state prediction for tk=2
for r = 1:ac
    for c = 1:ac
        if (rem(r, ac) == 0)
            X2(r, c) = X1(r/ac, ac) + (V1(r / ac, ac) * tk) + (0.5 * A(2, c) * tk * tk);
            PX2(r, c) = PX1(r/ac, ac) * PA(2, c);
            V2(r, c) = V1(r/ac, ac) + (A(2, c) * tk);
        else if (rem(r, ac) == 1)
                X2(r, c) = X1(((r + 4)/ac), ac-4) + (V1(((r + 4)/ac), ac - 4) * tk) + (0.5 * A(2, c) * tk * tk);
                PX2(r, c) = PX1(((r + 4)/ac), ac-4) * PA(2, c);
                V2(r, c) = V1(((r + 4)/ac), ac-4) + (A(2, c) * tk);
            else if (rem(r, ac) == 2)
                    X2(r, c) = X1(((r + 3)/ac), ac-3) + (V1(((r + 3)/ac), ac - 3) * tk) + (0.5 * A(2, c) * tk * tk);
                    PX2(r, c) = PX1(((r + 3)/ac), ac-3) * PA(2, c);
                    V2(r, c) = V1(((r + 3)/ac), ac-3) + (A(2, c) * tk);
                else if (rem(r, ac) == 3)
                        X2(r, c) = X1(((r + 2)/ac), ac-2) + (V1(((r + 2)/ac), ac - 2) * tk) + (0.5 * A(2, c) * tk * tk);
                        PX2(r, c) = PX1(((r + 2)/ac), ac-2) * PA(2, c);
                        V2(r, c) = V1(((r + 2)/ac), ac-2) + (A(2, c) * tk);
                    else if (rem(r, ac) == 4)
                            X2(r, c) = X1(((r + 1)/ac), ac-1) + (V1(((r + 1)/ac), ac - 1) * tk) + (0.5 * A(2, c) * tk * tk);
                            PX2(r, c) = PX1(((r + 1)/ac), ac-1) * PA(2, c);
                            V2(r, c) = V1(((r + 1)/ac), ac-1) + (A(2, c) * tk);
                        end
                    end
                end
            end
        end
    end
end


%state prediction for tk=3
for r = 1:ac * ac
    for c = 1:ac
        if (rem(r, ac) == 0)
            X3(r, c) = X2(r/ac, ac) + (V2(r / ac, ac) * tk) + (0.5 * A(3, c) * tk * tk);
            PX3(r, c) = PX2(r/ac, ac) * PA(3, c);
            V3(r, c) = V2(r/ac, ac) + (A(3, c) * tk);
        else if (rem(r, ac) == 1)
                X3(r, c) = X2(((r + 4)/ac), ac-4) + (V2(((r + 4)/ac), ac - 4) * tk) + (0.5 * A(3, c) * tk * tk);
                PX3(r, c) = PX2(((r + 4)/ac), ac-4) * PA(3, c);
                V3(r, c) = V2(((r + 4)/ac), ac-4) + (A(3, c) * tk);
            else if (rem(r, ac) == 2)
                    X3(r, c) = X2(((r + 3)/ac), ac-3) + (V2(((r + 3)/ac), ac - 3) * tk) + (0.5 * A(3, c) * tk * tk);
                    PX3(r, c) = PX2(((r + 3)/ac), ac-3) * PA(3, c);
                    V3(r, c) = V2(((r + 3)/ac), ac-3) + (A(3, c) * tk);
                else if (rem(r, ac) == 3)
                        X3(r, c) = X2(((r + 2)/ac), ac-2) + (V2(((r + 2)/ac), ac - 2) * tk) + (0.5 * A(3, c) * tk * tk);
                        PX3(r, c) = PX2(((r + 2)/ac), ac-2) * PA(3, c);
                        V3(r, c) = V2(((r + 2)/ac), ac-2) + (A(3, c) * tk);
                    else if (rem(r, ac) == 4)
                            X3(r, c) = X2(((r + 1)/ac), ac-1) + (V2(((r + 1)/ac), ac - 1) * tk) + (0.5 * A(3, c) * tk * tk);
                            PX3(r, c) = PX2(((r + 1)/ac), ac-1) * PA(3, c);
                            V3(r, c) = V2(((r + 1)/ac), ac-1) + (A(3, c) * tk);
                        end
                    end
                end
            end
        end
    end
end

%state prediction for tk=4
for r = 1:ac * ac * ac
    for c = 1:ac
        if (rem(r, ac) == 0)
            X4(r, c) = X3(r/ac, ac) + (V3(r / ac, ac) * tk) + (0.5 * A(4, c) * tk * tk);
            PX4(r, c) = PX3(r/ac, ac) * PA(4, c);
            V4(r, c) = V3(r/ac, ac) + (A(4, c) * tk);
        else if (rem(r, ac) == 1)
                X4(r, c) = X3(((r + 4)/ac), ac-4) + (V3(((r + 4)/ac), ac - 4) * tk) + (0.5 * A(4, c) * tk * tk);
                PX4(r, c) = PX3(((r + 4)/ac), ac-4) * PA(4, c);
                V4(r, c) = V3(((r + 4)/ac), ac-4) + (A(4, c) * tk);
            else if (rem(r, ac) == 2)
                    X4(r, c) = X3(((r + 3)/ac), ac-3) + (V3(((r + 3)/ac), ac - 3) * tk) + (0.5 * A(4, c) * tk * tk);
                    PX4(r, c) = PX3(((r + 3)/ac), ac-3) * PA(4, c);
                    V4(r, c) = V3(((r + 3)/ac), ac-3) + (A(4, c) * tk);
                else if (rem(r, ac) == 3)
                        X4(r, c) = X3(((r + 2)/ac), ac-2) + (V3(((r + 2)/ac), ac - 2) * tk) + (0.5 * A(4, c) * tk * tk);
                        PX4(r, c) = PX3(((r + 2)/ac), ac-2) * PA(4, c);
                        V4(r, c) = V3(((r + 2)/ac), ac-2) + (A(4, c) * tk);
                    else if (rem(r, ac) == 4)
                            X4(r, c) = X3(((r + 1)/ac), ac-1) + (V3(((r + 1)/ac), ac - 1) * tk) + (0.5 * A(4, c) * tk * tk);
                            PX4(r, c) = PX3(((r + 1)/ac), ac-1) * PA(4, c);
                            V4(r, c) = V3(((r + 1)/ac), ac-1) + (A(4, c) * tk);
                        end
                    end
                end
            end
        end
    end
end

%X prediction for tk=5
for r = 1:ac * ac * ac * ac
    for c = 1:ac
        if (rem(r, ac) == 0)
            X5(r, c) = X4(r/ac, ac) + (V4(r / ac, ac) * tk) + (0.5 * A(5, c) * tk * tk);
            PX5(r, c) = PX4(r/ac, ac) * PA(5, c);
            V5(r, c) = V4(r/ac, ac) + (A(5, c) * tk);
        else if (rem(r, ac) == 1)
                X5(r, c) = X4(((r + 4)/ac), ac-4) + (V4(((r + 4)/ac), ac - 4) * tk) + (0.5 * A(5, c) * tk * tk);
                PX5(r, c) = PX4(((r + 4)/ac), ac-4) * PA(5, c);
                V5(r, c) = V4(((r + 4)/ac), ac-4) + (A(5, c) * tk);
            else if (rem(r, ac) == 2)
                    X5(r, c) = X4(((r + 3)/ac), ac-3) + (V4(((r + 3)/ac), ac - 3) * tk) + (0.5 * A(5, c) * tk * tk);
                    PX5(r, c) = PX4(((r + 3)/ac), ac-3) * PA(5, c);
                    V5(r, c) = V4(((r + 3)/ac), ac-3) + (A(5, c) * tk);
                else if (rem(r, ac) == 3)
                        X5(r, c) = X4(((r + 2)/ac), ac-2) + (V4(((r + 2)/ac), ac - 2) * tk) + (0.5 * A(5, c) * tk * tk);
                        PX5(r, c) = PX4(((r + 2)/ac), ac-2) * PA(5, c);
                        V5(r, c) = V4(((r + 2)/ac), ac-2) + (A(5, c) * tk);
                    else if (rem(r, ac) == 4)
                            X5(r, c) = X4(((r + 1)/ac), ac-1) + (V4(((r + 1)/ac), ac - 1) * tk) + (0.5 * A(5, c) * tk * tk);
                            PX5(r, c) = PX4(((r + 1)/ac), ac-1) * PA(5, c);
                            V5(r, c) = V4(((r + 1)/ac), ac-1) + (A(5, c) * tk);
                        end
                    end
                end
            end
        end
    end
end
% X prediction for expected trajectory 
XC=zeros(1,ac);
for i=1:(ac+1)
    if i==1
       XC(1,i)=X0(1,i); 

    else
        XC(1,i)=X0(1,1)+V0(1,1)*(i-1)*tk;
    end
end
end
