clear all;
DATASET = xlsread('roadBoundaryEncounter.xlsx');
y = DATASET(:, 1);
length = size(DATASET, 1);
% the xlsx file contain one column as follows
% Y: Lateral distance of subject from the lane marking in m
W = 4; % vehicle width;
k2 = 14; % model parameter
k = 0.61; % model parameter
vy(1, 1) = 0;
prob(1, 1) = 0;

for i = 2:length
    vy(i, 1) = (y(i, 1) - y(i - 1, 1)) / 0.1;
    prob(i, 1) = exp(-k2*y(i, 1)/W);

end
svy = smooth(vy, 10);
for i = 1:length
    ts(i, 1) = i * 0.1;
    if svy(i, 1) < 0
        sev(i, 1) = 0.5 * 1000 * (svy(i, 1)^2) * k;
        tlc(i, 1) = -(y(i, 1)) / svy(i, 1); %-svy(i,1)/(y(i,1));
    else
        sev(i, 1) = 0;
        tlc(i, 1) = inf;
    end

    risk(i, 1) = prob(i, 1) * sev(i, 1);
end

figure;
subplot(3, 2, 1)
plot(ts, y, '-k', 'LineWidth', 2)
set(gca, 'fontsize', 10)
legend(['Lateral spacing (m)'])
xlabel({'[a] Time (s)'});

subplot(3, 2, 2)
set(gca, 'fontsize', 10)
plot(ts, sev, '-r', 'LineWidth', 2)
legend(['Severity in Joules'])
xlabel({'[b] Time (s)'});

subplot(3, 2, 3)
set(gca, 'fontsize', 10)
plot(ts, prob, '-r', 'LineWidth', 2)
legend(['Collision probability'])
xlabel({'[c] Time (s)'});

subplot(3, 2, 4)
set(gca, 'fontsize', 10)
plot(ts, risk, '-r', 'LineWidth', 2)
legend(['PDRF risk (Joules)'])
xlabel({'[d] Time (s)'});

subplot(3, 2, 5)
set(gca, 'fontsize', 10)
plot(ts, tlc, '-r', 'LineWidth', 2)
ylim([0, 10]);
xlim([0, 6]);
legend(['Time to Lane Crossing (s)'])
xlabel({'[e] Time (s)'});

subplot(3, 2, 6)
set(gca, 'fontsize', 10)
plot(ts, svy, '-k', 'LineWidth', 2)
legend(['Lateral velocity (m/s)'])
xlabel({'[f] Time (s)'});


